//
//  LiveRoom-Publisher.h
//  zegoliveroom
//
//  Created by Randy Qiu on 2017/1/23.
//
//

#ifndef LiveRoom_Publisher_h
#define LiveRoom_Publisher_h

#include "./LiveRoomDefines-Publisher.h"
#include "./LiveRoomCallback-Publisher.h"
#include "./LiveRoomDefines.h"

namespace AVE
{
    class VideoCaptureFactory;
    class MediaCaptureFactory;
    class VideoFilterFactory;
    struct AudioFrame;
    struct ExtAudioProcSet;
    typedef ExtAudioProcSet ExtPrepSet;

    /**
     音频处理回调函数定义

     @param inFrame 待处理音频数据
     @param outFrame 处理后的音频数据，其中buffer 所指向的内存由 SDK 负责申请和销毁，用户处理后回写的数据的采样率不超过48K
     @attention 请确保在当前线程完成，且不要做耗时操作
     */
    typedef void (*OnAudioProcCallback)(const AudioFrame& inFrame, AudioFrame& outFrame);

    typedef OnAudioProcCallback OnPrepCallback;
}

namespace ZEGO
{
    namespace LIVEROOM
    {
		/**
		获取 SDK 支持的最大同时支持的推流路数

		@return 最大支持推流路数
		*/
        ZEGO_API int GetMaxPublishChannelCount();

        /**
         设置直播主播相关信息通知的回调

         @param pCB 回调对象指针
         @return true 成功，false 失败
         */
        ZEGO_API bool SetLivePublisherCallback(ILivePublisherCallback* pCB);
        
        /**
         设置本地预览视图

         @param pView 用于渲染本地预览视频的视图
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool SetPreviewView(void* pView, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         启动本地预览
         
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 启动本地预览前，要调用 LIVEROOM::SetPreviewView 设置本地预览视图
         */
        ZEGO_API bool StartPreview(AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         结束本地预览
         
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 建议停止推流，或本地预览结束后，调用该 API 停止本地预览
         */
        ZEGO_API bool StopPreview(AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置或更新推流的附加信息

         @param pszStreamExtraInfo 流附加信息, 最大为 1024 字节
         @param idx 推流 channel Index. 默认为主Channel
         @return 更新流附加信息成功后，同一房间内的其他人会收到 OnStreamExtraInfoUpdated 通知
         */
        ZEGO_API bool SetPublishStreamExtraInfo(const char *pszStreamExtraInfo, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         自定义转推目的地

         @param pszCustomPublishTarget 目的 rmtp 推流地址
         @param idx 推流 channel Index. 默认为主Channel
         */
        ZEGO_API void SetCustomPublishTarget(const char *pszCustomPublishTarget, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         添加转推地址
         
         @param strTarget 转推地址（支持rtmp/avertp）
         @param pszStreamID 推流ID
         @attention 在InitSDK之后调用
         */
        ZEGO_API int AddPublishTarget(const char *strTarget, const char* pszStreamID);
        
        /**
         删除转推地址
         
         @param strTarget 转推地址（支持rtmp/avertp）
         @param pszStreamID 推流ID
         @attention 在InitSDK之后调用
         */
        ZEGO_API int DeletePublishTarget(const char *strTarget, const char* pszStreamID);
        
        /**
         单主播模式下，自定义直推CDN的地址
         
         @param pszCDNPublishTarget 目的 rtmp 推流地址
         @param idx 推流 channel Index. 默认为主Channel
         */
        ZEGO_API void SetCDNPublishTarget(const char *pszCDNPublishTarget, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置推流加密密钥
         
         @param pszKey 加密密钥
         @param nkeyLen 密钥长度（支持16/24/32字节）
         @param idx 推流 channel Index. 默认为主Channel
         @attention 在InitSDK之后调用
         */
        ZEGO_API void SetPublishEncryptKey(const unsigned char *pKey, int nkeyLen, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
          设置采集视频源类型

          @note 
            1. 在推流前调用，推流过程中不能更改此值，否则可能会出现不可预知问题; 
            2. 当 srcType 设置为 VIDEO_SRC_EXTERNAL_CAPTURE 但对应通道没有外部采集工厂时，将自动修正为 VIDEO_SRC_CAMERA;
            3. 当 srcType 设置为 VIDEO_SRC_NONE 时，表示不启动视频采集，即没有视频数据;
            4. 当 srcType 设置为 VIDEO_SRC_DEFAULT 时，如果有设置外部视频采集工厂，则使用外部采集，否则使用内部采集;
            5. 当 srcType 设置为 VIDEO_SRC_CAMERA 时，即使设置了外部视频采集工厂，也会使用摄像头作为视频数据源;
            6. 当 index 为 PUBLISH_CHN_MAIN 时，srcType 不能设置为 VIDEO_SRC_MAIN_PUBLISH_CHN;
            7. 一旦使用过此 API 设置采集视频源类型，在停止推流时，SDK 不会重置视频采集源类型;

         @param srcType 源类型，参见 ZegoVideoSourceType 定义
         @param idx 推流 channel Index. 默认为辅Channel
         @return true 设置成功，false 设置失败
         */
        ZEGO_API bool SetVideoSource(AV::ZegoVideoSourceType srcType, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_AUX);

        /**
         开始直播

         @attention 
          1. 调用此 API 推直播流，必须在调用初始化 SDK 接口之后调用。
          2. 不支持多房间模式（通过 SetRoomMode 接口开启 ZegoRoomModeMultiRoom）。

         @note
          1. 推流成功后，等待 ILivePublisherCallback::OnPublishStateUpdate 回调
          2. 若中途收到 IRoomCallback::OnDisconnect 回调，则不会再收到 ILivePublisherCallback::OnPublishStateUpdate 回调。

         @param pszTitle 直播名称
         @param pszStreamID 流 ID
         @param flag 直播属性，参考 ZegoPublishFlag
         @param pszParams 推流参数
         @return true 成功，false 失败
         */
        ZEGO_API bool StartPublishing(const char* pszTitle, const char* pszStreamID, int flag, const char* pszParams = 0);
        
        /**
         开始直播

         @attention 
          1. 调用此 API 推直播流，必须在调用初始化 SDK 接口之后调用。
          2. 不支持多房间模式（通过 SetRoomMode 接口开启 ZegoRoomModeMultiRoom）。

         @note 
          1. 推流成功后，等待 ILivePublisherCallback::OnPublishStateUpdate 回调
          2. 若中途收到 IRoomCallback::OnDisconnect 回调，则不会再收到 ILivePublisherCallback::OnPublishStateUpdate 回调
          3. 调用此接口 SetMixStreamConfig 无效，混流需要调用 MixStreamEx。
         
         @param pszTitle 直播名称
         @param pszStreamID 流 ID
         @param flag 直播属性，参考 ZegoPublishFlag
         @param pszParams 推流参数
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool StartPublishing2(const char* pszTitle, const char* pszStreamID, int flag, const char* pszParams = 0, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         开始直播
         
         @attention 
          1. 调用此 API 推直播流，必须在调用初始化 SDK 接口之后调用。
          2. 支持多房间模式（通过 SetRoomMode 接口开启 ZegoRoomModeMultiRoom），必须明确指定房间 ID。
         
         @note 
          1. 推流成功后，等待 ILivePublisherCallback::OnPublishStateUpdate 回调
          2. 若中途收到 IRoomCallback::OnDisconnect 回调，则不会再收到 ILivePublisherCallback::OnPublishStateUpdate 回调
          3. 调用此接口 SetMixStreamConfig 无效，混流需要调用 MixStreamEx。
         
         @param streamParams 推流参数
         @return true 成功，false 失败
         */
        ZEGO_API bool StartPublishingWithParams(const ZegoPublishStreamParams* streamParams);
    
        /**
         推流时是否发送视频数据。

         @param mute true 不发送（仅预览），false 发送
         @param idx 推流通道索引. 默认为主Channel
         @return 0 代表设置成功成功，否则设置失败
         @attention 拉流端通过 OnRemoteCameraStatusUpdate 回调监听此状态是否改变;
         @attention 仅拉 UDP 流时，才能接收到状态变更通知;
        */
        ZEGO_API int MuteVideoPublish(bool mute, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         推流时是否发送音频数据。
         
         @param mute true 不发送，false 发送
         @param idx 推流通道索引. 默认为主Channel
         @return 0 代表设置成功成功，否则设置失败
         @attention 可以通过 OnAudioRecordCallback 回调获取本地音频数据
         */
        ZEGO_API int MuteAudioPublish(bool mute, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         停止直播

         @param flag 保留字段
         @param pszMsg 自定义信息，server 对接流结束回调包含此字段内容
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool StopPublishing(int flag = 0, const char* pszMsg = 0, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         回应连麦申请

         @param seq 收到的连麦申请 seq (通过 ILivePublisherCallback::OnJoinLiveRequest 收到申请)
         @param rspResult 回应，0 表示同意
         @param pszRoomID 房间 ID
         @return true 成功，false 失败
         @note 申请者会收到 OnJoinLiveResponse 回调
         @attention 单房间模式，设置 roomID 为当前房间或者为空，都往当前房间发送。
         @attention 多房间模式（通过 SetRoomMode 接口开启 ZegoRoomModeMultiRoom），必须指定 roomID，否则调用失败。
         */
        ZEGO_API bool RespondJoinLiveReq(int seq, int rspResult, const char* pszRoomID = nullptr);
        
        /**
         邀请连麦

         @param pszUserID 准备邀请的用户 ID
         @param pszRoomID 房间 ID
         @return 请求 seq，正值为成功，其他为失败
         @attention 邀请成功后，等待 ILivePublisherCallback::OnInviteJoinLiveResponse 回调
         @attention 单房间模式，设置 roomID 为当前房间或者为空，都往当前房间发送。
         @attention 多房间模式（通过 SetRoomMode 接口开启 ZegoRoomModeMultiRoom），必须指定 roomID，否则调用失败。
         @note 被邀请的用户会收到 OnInviteJoinLiveRequest 回调
         */
        ZEGO_API int InviteJoinLive(const char* pszUserID, const char* pszRoomID = nullptr);

        /**
         结束连麦

         @param pszUserID 指定结束连麦的用户 ID
         @param pszRoomID 房间 ID
         @return 请求seq，正值为有效，等待 ILivePublisherCallback::OnEndJoinLive 回调
         @attention 单房间模式，设置 roomID 为当前房间或者为空，都往当前房间发送。
         @attention 多房间模式（通过 SetRoomMode 接口开启 ZegoRoomModeMultiRoom），必须指定 roomID，否则调用失败。
         */
        ZEGO_API int EndJoinLive(const char* pszUserID, const char* pszRoomID = nullptr);
        
        /**
         开启/关闭推流硬件编码。

         @attention 
         * 必须在推流前设置；
         * 在反初始化 SDK 前，设置的值一直有效。

         @param bRequired true 开启，false 关闭。默认 false
         @return true 成功，false 失败
         @note 打开硬编硬解开关需后台可控，避免碰到版本升级或者硬件升级时出现硬编硬解失败的问题；
         @note 如果要单独控制某个推流通道，请参考：RequireHardwareEncoderByChannel；
         @see RequireHardwareEncoderByChannel
         */
        ZEGO_API bool RequireHardwareEncoder(bool bRequired);

        /**
         为某个推流道通单独开启/关闭硬件编码。

         @attention 
         * 必须在推流前设置；
         * 在反初始化 SDK 前，设置的值一直有效。

         @param enable true 开启，false 关闭。如果有调用 RequireHardwareEncoder 设置，则默认为 RequireHardwareEncoder 设置的值，否则默认为 false
         @param idx 推流通道
         @return true 成功，false 失败
         @note 打开硬编硬解开关需后台可控，避免碰到版本升级或者硬件升级时出现硬编硬解失败的问题；
         @note 如果想统一控制所有通道的硬编码开关，请参考：RequireHardwareEncoder；
         @note 通过此 API 设置的值会与通过 RequireHardwareEncoder 设置的值相互覆盖，具体取决于其调用顺序。
         @see RequireHardwareEncoder
         */
        ZEGO_API bool RequireHardwareEncoderByChannel(bool enable, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         设置视频码率

         @param nBitrate 码率，单位为bps
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool SetVideoBitrate(int nBitrate, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         设置视频帧率

         @param nFps 帧率
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @note 推流端设置60帧，拉流端生效需要联系技术支持
         */
        ZEGO_API bool SetVideoFPS(int nFps, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置视频编码输出分辨率

         @param nWidth 宽
         @param nHeight 高
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool SetVideoEncodeResolution(int nWidth, int nHeight, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置视频采集分辨率

         @param nWidth 宽
         @param nHeight 高
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool SetVideoCaptureResolution(int nWidth, int nHeight, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置视频关键帧间隔
         
         @param nIntervalSecond 关键帧间隔，单位为秒，默认2秒
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流开始前调用本 API 进行参数配置
         */
        ZEGO_API bool SetVideoKeyFrameInterval(int nIntervalSecond, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         主播开启美颜功能

         @param nFeature 美颜特性。默认无美颜
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置
         */
        ZEGO_API bool EnableBeautifying(int nFeature, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置美颜磨皮的采样步长

         @param step 采样步长，取值范围[1,16]。默认 4.0
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置。设置时需确保对应美颜特性开启
         */
        ZEGO_API bool SetPolishStep(float step, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置美颜采样颜色阈值

         @param factor 采样颜色阈值，取值范围[0,16]。默认 4.0
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置。设置时需确保对应美颜特性开启
         */
        ZEGO_API bool SetPolishFactor(float factor, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置美颜美白的亮度修正参数

         @param factor 亮度修正参数，取值范围[0,1]，值越大亮度越暗。默认 0.5
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置。设置时需确保对应美颜特性开启
         */
        ZEGO_API bool SetWhitenFactor(float factor, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置锐化参数

         @param factor 锐化参数，取值范围[0,2]，值越大锐化越强。默认 0.2
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置。设置时需确保对应美颜特性开启
         */
        ZEGO_API bool SetSharpenFactor(float factor, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置滤镜

         @param nIndex 滤镜索引。默认不使用滤镜
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置
         */
        ZEGO_API bool SetFilter(int nIndex, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置本地预览视频视图的模式

         @param mode 模式
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流开始前调用本 API 进行参数配置
         */
        ZEGO_API bool SetPreviewViewMode(ZegoVideoViewMode mode, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         是否启用预览镜像
         
         @param bEnable true 启用，false 不启用。默认 true
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @note 默认启用预览镜像
         */
        ZEGO_API bool EnablePreviewMirror(bool bEnabasle, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         是否启用预览和推流镜像
         
         @param mode 镜像模式
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @note 默认启用预览镜像，不启用推流镜像
         */
        ZEGO_API bool SetVideoMirrorMode(AV::ZegoVideoMirrorMode mode, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置预览视频控件的背景颜色
         
         @param  color 颜色,取值为0x00RRGGBB
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool SetPreviewViewBackgroundColor(int color, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置预览渲染朝向

         @param nRotation 旋转角度(0/90/180/270)。默认 0
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置
         */
        ZEGO_API bool SetPreviewRotation(int nRotation, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
        设置视频采集旋转角度

        @param nRotation 逆时针旋转角度(0/90/180/270)。默认 0
        @param idx 推流 channel Index. 默认为主Channel
        @return true 成功，false 失败
        @attention 在InitSDK之后调用有效，uninitSDK前有效。支持在推流时调用。
        */
        ZEGO_API bool SetCaptureFrameRotation(int nRotation, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         开启麦克风

         @param bEnable true 打开，false 关闭。默认 true
         @return true 成功，false 失败
         @attention 推流时可调用本 API 进行参数配置
         */
        ZEGO_API bool EnableMic(bool bEnable);
        
        /**
         设置音频码率

         @param bitrate 码率
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool SetAudioBitrate(int bitrate, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置音频设备模式

         @param mode 模式, 默认 ZEGO_AUDIO_DEVICE_MODE_AUTO
         @attention 在推流预览前后调用均有效，调用该接口会触发设备的启动切换，建议不要频繁调用，避免不必要的开销与硬件问题
         @attention 调用本接口可能导致音量模式在 通话/媒体 间切换，若媒体音量和通话音量不一致，可能导致音量变化
         */
        ZEGO_API void SetAudioDeviceMode(AV::ZegoAVAPIAudioDeviceMode mode);
        
        /**
         音频采集自动增益开关。windows默认开启；android/ios默认关闭；mac默认音频设备模式为ZEGO_AUDIO_DEVICE_MODE_COMMUNICATION或ZEGO_AUDIO_DEVICE_MODE_COMMUNICATION2时开启，否则关闭

         @param bEnable true 开启，false 关闭
         @return true 成功，false 失败
         @discussion 建议在推流前调用设置
         */
        ZEGO_API bool EnableAGC(bool bEnable);

        /**
         回声消除开关

         @param bEnable true 开启，false 关闭
         @return true 成功，false 失败
         @discussion 建议在推流前调用设置
         */
        ZEGO_API bool EnableAEC(bool bEnable);
        
        
        /**
         设置回声消除模式

         @param mode 回声消除模式
         @discussion 建议在推流前调用设置
         */
        ZEGO_API void SetAECMode(AV::ZegoAECMode mode);
        
        /**
         开启摄像头

         @param bEnable true 开启，false 关闭。默认 true
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，false 失败
         */
        ZEGO_API bool EnableCamera(bool bEnable, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         截预览图
         
         @param idx 推流 channel Index. 默认为主Channel
         @return true 成功，通过回调返回结果，false 失败
         */
        ZEGO_API bool TakeSnapshotPreview(AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         开启采集监听

         @param bEnable true 打开，false 关闭。默认 false
         @return true 成功，false 失败
         @discussion 推流时可调用本 API 进行参数配置。连接耳麦时设置才实际生效。开启采集监听，主播方讲话后，会听到自己的声音。
         */
        ZEGO_API bool EnableLoopback(bool bEnable);
        
        /**
         设置监听音量

         @param volume 音量大小，取值范围 [0, 200], 默认 60
         @attention 推流时可调用本 API 进行参数配置
         */
        ZEGO_API void SetLoopbackVolume(int volume);
        
        /**
         设置采集音量
         
         @param volume 音量大小，取值范围 [0, 200], 默认 100
         @attention SDK初始化成功后调用
         */
        ZEGO_API void SetCaptureVolume(int volume);
        
        /**
         获取当前采集的音量

         @return 当前采集音量大小
         */
        ZEGO_API float GetCaptureSoundLevel();

        /**
         设置关闭摄像头时所推静态图片的路径
         
         @param filePath 图片路径
         @param idx 推流 channel Index. 默认为主Channel
         @attention InitSDK之后，关闭摄像头前调用本 API 进行参数配置，关闭摄像头后会推静态图片
         @attention 支持推静态图片时调用此 API 动态更换静态图片
         */
        ZEGO_API void SetDummyCaptureImagePath(const char *filePath, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置水印的图片路径

         @param filePath 图片路径
         @param idx 推流 channel Index. 默认为主Channel
         @attention 推流开始前调用本 API 进行参数配置
         */
        ZEGO_API void SetWaterMarkImagePath(const char *filePath, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置水印在采集视频中的位置

         @param left 左上角坐标的第一个元素
         @param top 左上角坐标的第二个元素，即左上角坐标为 (left, top)
         @param right 右下角坐标的第一个元素
         @param bottom 右下角坐标的第二个元素，即右下角坐标为 (right, bottom)
         @param idx 推流 channel Index. 默认为主Channel
         @note 左上角为坐标系原点，区域不能超过编码分辨率设置的大小
         */
        ZEGO_API void SetPublishWaterMarkRect(int left, int top, int right, int bottom, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         设置水印在预览视频中的位置

         @param left 左上角坐标的第一个元素
         @param top 左上角坐标的第二个元素，即左上角坐标为 (left, top)
         @param right 右下角坐标的第一个元素
         @param bottom 右下角坐标的第二个元素，即右下角坐标为 (right, bottom)
         @param idx 推流 channel Index. 默认为主Channel
         @note 左上角为坐标系原点，区域不能超过 preview 的大小
         */
        ZEGO_API void SetPreviewWaterMarkRect(int left, int top, int right, int bottom, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置选用编码格式
         
         @param codecId 编码 ID
         @return true 成功，false 失败
         */
        ZEGO_API bool SetVideoCodecId(AV::ZegoVideoCodecAvc codecId, AV::PublishChannelIndex nChannel = AV::PUBLISH_CHN_MAIN);

        /**
         设置音频前处理函数(耳返前), 并开启/关闭音频前处理特性

         @param callback 音频前处理函数指针
         @param config 预处理的采样率等参数设置
         @attention 必须在InitSDK后，在调用预览，拉流，推流，启动播放器接口前设置
         @attention 该接口设置的回调处理音频影响耳返效果
         */
        ZEGO_API void SetAudioPrepCallback(AVE::OnPrepCallback callback, const AVE::ExtPrepSet& config);

        /**
        设置音频处理函数(耳返后), 并开启/关闭音频处理特性

        @param callback 音频处理函数指针
        @param config 预处理的采样率等参数设置
        @attention 必须在 InitSDK 后且在推流前调用
        @attention 该接口设置的回调处理音频不影响耳返效果
        */
        ZEGO_API void SetAudioPrepAfterLoopbackCallback(AVE::OnAudioProcCallback callback, const AVE::ExtPrepSet& config);
        
        /**
         设置编码器码率控制策略

         @param strategy 策略配置，参考 ZegoVideoEncoderRateControlStrategy
         @param encoderCRF 当策略为恒定质量（ZEGO_RC_VBR/ZEGO_RC_CRF）有效，取值范围 [0~51]，越小质量越好，但是码率会相应变大。建议取值范围 [18, 28]
         @param idx 推流 channel Index. 默认为主Channel
         */
        ZEGO_API void SetVideoEncoderRateControlConfig(AV::ZegoVideoEncoderRateControlStrategy strategy, int encoderCRF, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         帧顺序检测开关
         
         @param bEnalbe true 检测帧顺序，不支持B帧； false 不检测帧顺序，支持B帧，可能出现短暂花屏
         */
        ZEGO_API void EnableCheckPoc(bool bEnable);
        
        /**
         设置视频采集缩放时机
         
         @param mode 视频采集缩放时机，请参考 AV::ZegoCapturePipelineScaleMode 定义。默认为 ZegoCapturePipelinePreScale
         @discussion 初始化 SDK 后，StartPreview, StartPublish 前调用。摄像头启动之后设置不会立即生效，而是在下次摄像头启动时生效。
         */
        ZEGO_API void SetCapturePipelineScaleMode(AV::ZegoCapturePipelineScaleMode mode);
        
        /**
         设置所有推流通道的延迟模式
         
         @param mode 延迟模式，默认 ZEGO_LATENCY_MODE_NORMAL
         @attention 确保在InitSDK后，推流前调用
         */
        ZEGO_API void SetLatencyMode(AV::ZegoAVAPILatencyMode mode);
        
        /**
         设置指定推流通道的延迟模式

         @param mode 延迟模式，默认 ZEGO_LATENCY_MODE_NORMAL
         @param idx 推流 channel Index. 详见 AV::PublishChannelIndex
         @attention 确保在InitSDK后，推流前调用
         */
        ZEGO_API void SetLatencyModeByChannel(AV::ZegoAVAPILatencyMode mode, AV::PublishChannelIndex idx);
        
        /**
         设置所有推流通道的推流音频声道数
         
         @param count 声道数，1 或 2，默认为 1（单声道）
         @attention 必须在InitSDK后，推流前设置
         @note SetLatencyMode 设置为 ZEGO_LATENCY_MODE_NORMAL, ZEGO_LATENCY_MODE_NORMAL2, ZEGO_LATENCY_MODE_LOW3 才能设置双声道
         @note 在移动端双声道通常需要配合音频前处理才能体现效果。
         */
        ZEGO_API void SetAudioChannelCount(int count);
        
        /**
         设置指定推流通道的推流音频声道数
         
         @param count 声道数，1 或 2，默认为 1（单声道）
         @param idx 推流 channel Index. 详见 AV::PublishChannelIndex
         @attention 必须在InitSDK后，推流前设置
         @note SetLatencyMode 设置为 ZEGO_LATENCY_MODE_NORMAL, ZEGO_LATENCY_MODE_NORMAL2, ZEGO_LATENCY_MODE_LOW3 才能设置双声道
         @note 在移动端双声道通常需要配合音频前处理才能体现效果。
         */
        ZEGO_API void SetAudioChannelCountByChannel(int count, AV::PublishChannelIndex idx);
        
        /**
         是否开启离散音频包发送

         @param bEnable true 开启，推流中检测为静音的数据包不发送；false 关闭，推流中的静音数据包照常发送。默认是 false。
         @attention 推流前调用有效。以 RTC 方式推流，此接口才会生效。
         @attention SDK 默认关闭该特性。
         @note 可以和 EnableVAD 配合使用。
         @note 开启离散音频包发送可以在关闭麦克风或者静音的状态下，停止音频包的发送，节省用户流量。
         @note 如果为音频单流(非混流)转推 CDN，开启此选项将导致 CDN 在设定时间内未收到数据包认为流关闭而断开连接，所以此种情况不建议开启。
         */
        ZEGO_API void EnableDTX(bool bEnable);
        
        /**
         是否开启语音活动检测
         
         @param bEnable true 开启；false 关闭，默认关闭
         @attention 确保在推流前调用，只有纯 UDP 方案才可以调用此接口
         */
        ZEGO_API void EnableVAD(bool bEnable);
    
        /**
         是否开启流量控制

         @param properites 流量控制属性 (帧率，分辨率），参考 ZegoTrafficControlProperty 定义。默认 ZEGO_TRAFFIC_CONTROL_ADAPTIVE_FPS
         @param bEnable true 开启，false 关闭。默认开启。
         @param idx 推流通道，参考 AV::PublishChannelIndex 定义。默认 AV::PUBLISH_CHN_MAIN。
         @note bEnable 设置为 false 时，properties 参数无效。
         @note 在初始化 SDK 之后，推流之前调用，如果每次推流前不重新设置，将继续使用上次的配置，在反初始化 SDK 之后失效。
         @note 只适用于 UDP 推流。
         @note 默认开启主通道, 关闭其他通道。
         */
        ZEGO_API void EnableTrafficControl(int properties, bool bEnable, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置推流视频码率达不到最小阈值时的控制策略

         @param nBitrate 视频码率最小阈值，单位为bps。
         @param mode 低于视频码率最小阈值时的视频发送模式。当网络情况较差，无法发送该最小码率的数据时，如果选择 ZEGO_TRAFFIC_CONTROL_MIN_VIDEO_BITRATE_NO_VIDEO 模式，视频会被暂停，而不是以低于该码率继续发送。如果选择 ZEGO_TRAFFIC_CONTROL_MIN_VIDEO_BITRATE_ULTRA_LOW_FPS 模式，视频会以超低帧率发送。
         @param idx 推流通道，参考 AV::PublishChannelIndex 定义。默认 AV::PUBLISH_CHN_MAIN。
         @note 在初始化 SDK 之后调用有效，在反初始化 SDK 之后失效。
         @note 调用 EnableTrafficControl 接口启动该通道流量控制后，调用该接口才生效。
         @note 初始化 SDK 后默认情况下没有设置视频码率最小值，即尽可能的保持视频流畅。
         @note 如果需要取消该设置值的限制可以设置为 0。
        */
        ZEGO_API void SetMinVideoBitrateForTrafficControl(int nBitrate, AV::ZegoTrafficControlMinVideoBitrateMode mode = AV::ZEGO_TRAFFIC_CONTROL_MIN_VIDEO_BITRATE_NO_VIDEO, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
        
        /**
         设置触发流量控制的关注因素
         
         @param focusOn 触发流量控制的关注因素。选择 ZEGO_TRAFFIC_CONTROL_FOCUS_ON_LOCAL_ONLY 时，只关注本地网络状况。选择 ZEGO_TRAFFIC_CONTROL_FOCUS_ON_REMOTE 时，不仅关注本地网络，同时也兼顾远端网络，目前只在 1v1 场景下有效。
         @param idx 推流通道，参考 AV::PublishChannelIndex 定义。默认 AV::PUBLISH_CHN_MAIN。
         @note 在初始化 SDK 之后，推流之前调用有效。如果每次推流前不重新设置，将继续使用上次的配置。在反初始化 SDK 之后失效。
         @note 当 focusOn 设置为 ZEGO_TRAFFIC_CONTROL_FOCUS_ON_REMOTE 时，当对方的网络拥塞时，推流端可能限定推流的码率。
         @note 当通过 EnableTrafficControl 接口开启了指定推流通道的流量控制后，可通过该接口控制是否因为远端网络状况差而启动流量控制。在 1v1 场景下默认是开启的。
         */
        ZEGO_API void SetTrafficControlFocusOn(enum AV::ZegoTrafficControlFocusOn focusOn, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
    
        /**
        音频采集噪声抑制开关
         
        @param bEnalbe true 开启，false 关闭
        @return true 调用成功，false 调用失败
         */
        ZEGO_API bool EnableNoiseSuppress(bool bEnable);

        /**
          设置音频采集降噪等级
          @param mode 降噪等级，详见 ZegoANSMode 定义
          @return true 成功，false 失败
          @note 仅在 EnableNoiseSuppress 为 true 时有效, 默认为 MEDIUM
         */
        ZEGO_API bool SetNoiseSuppressMode(AV::ZegoANSMode mode);

        /**
         音频采集的瞬态噪声抑制开关（消除键盘、敲桌子等瞬态噪声）

         @param bEnable true 开启，false 关闭
         @return true 调用成功，false 调用失败
         */
        ZEGO_API bool EnableTransientNoiseSuppress(bool bEnable);

        /**
         设置推流质量监控周期
         
         @param timeInMS 时间周期，单位为毫秒，取值范围为(500, 60000)。默认为 3000
         @return true 成功，false 失败
         @attention 必须在推流前调用才能生效。该设置会影响 ILivePublisherCallback::OnPublishQualityUpdate 的回调频率
         */
        ZEGO_API bool SetPublishQualityMonitorCycle(unsigned int timeInMS);
        
        /**
         音效均衡器

         @param bandIndex 取值范围[0, 9]。分别对应10个频带，其中心频率分别是[31, 62, 125, 250, 500, 1K, 2K, 4K, 8K, 16K]Hz
         @param bandGain 取值范围[-15, 15]。默认值是0，如果所有频带的增益值全部为0，则会关闭EQ功能
         @return true 调用成功，false 调用失败
         @attention 在InitSDK之后调用有效。使用此接口前请与即构技术支持联系确认是否支持此功能
         @deprecated 请使用 AUDIOPROCESSING::SetAudioEqualizerGain(int, float)
         */
        ZEGO_API bool SetAudioEqualizerGain(int bandIndex, float bandGain);

        /**
         开启屏幕分享编码优化

         @param enable 是否开启。true：开启，false：关闭。默认是关闭的。
         @param idx 推流通道
         @attention 前置条件：仅在软件编码的情况下使用 OpenH264 编码时有效。在默认情况下 SDK 就是使用 OpenH264 编码器。
         @attention 调用 RequireHardwareAccelerated 或者 RequireHardwareEncoder开启硬件编码后，本接口失效。
         @attention 调用 SetVideoCodecId 设置非默认选项时，本接口失效。
         @note 初始化 SDK 之后，推流之前设置有效，当次推流结束后失效。
         @note 推荐使用场景：在屏幕共享推流时，SDK 会优化编码，使得清晰度会好很多，视频码率比较稳定，还支持分层编码。
         */
        ZEGO_API void EnableScreenCaptureEncodeOptimization(bool enable, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         给推流通道设置扩展参数，一般不建议修改

         @param param_config 参数配置信息
         @param idx 推流通道索引，默认主通道
         @attention 配置项写法，例如 "zego_channel_param_key_video_x264_config_tune=animation", 等号后面值的类型要看下面每一项的定义
         @attention "zego_channel_param_key_video_x264_config_tune", string value: animation, 设置编码的 tune 值，目前只支持 animation，仅使用 X264 编码时有效
         @attention 初始化 SDK 之后推流前设置才生效，推流过程中设置无效
         */
        ZEGO_API void SetChannelExtraParam(const char *param_config, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         获取推流通道扩展参数

         @param key 需要获取的参数类型，目前仅支持 AVCaptureDevice
         @param idx 推流通道索引，默认主通道

         @attention 初始化 SDK 之后调用
         @attention 目前仅支持 Mac/iOS 平台
         */
        ZEGO_API void* GetChannelExtraParam(AV::ChannelExtraParamKey key, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
        获取设备编码格式的能力，外部拿到信息 建议立即调用FreeVideoCodecCapabilityList 销毁SDK申请的内存。

        @param len 设备支持编码格式个数
        @return 设备编码格式的能力列表
        @attention 初始化 SDK 之后调用
        @attention 建议在回调初始化 SDK 成功之后调用该接口
        */
        ZEGO_API AV::ZegoCodecCapabilityInfo* GetVideoCodecCapabilityList(int& len);

        /**
        释放设备编码格式的能力列表

        @param parrCodecCapability 设备编码格式的能力列表
        */
        ZEGO_API void FreeVideoCodecCapabilityList(AV::ZegoCodecCapabilityInfo* parrCodecCapability);

        /**
         设置麦克风采集声音在混音过程中的偏移
         
         @param shiftMS 偏移值，当 shiftMS 小于 0 时，混音时会延迟播放音；当 shiftMS 大于 0 时，混音时会延迟采集音。有效值范围[-500, 500]，单位毫秒，默认值 0
         @attention 初始化 SDK 之后调用有效，推流前、推流过程中调用都有效
         @attention 只适用于 ktv 场景，偏移麦克风采集的人声，匹配播放器的伴奏声音
         @attention 当推流主通道采用音频内部采集时，对主通道推流，主通道本地录制有效
         */
        ZEGO_API void SetAudioCaptureShiftOnMix(int shiftMS);

        /**
         设置所有推流通道是否校准网络时间，用于混流时各路流的对齐
         
         @param enable true开启，false关闭
         @return 0 表示成功，-1 为失败
         @attention 此方法依赖网络时间模块
         @attention InitSDK后 推流之前设置  
         @deprecated 请使用 SetStreamAlignmentProperty 接口代替
         */
        ZEGO_API int EnablePublishStreamAlignment(bool enable);
    
        /**
         开/关 H.265 编码自动降级到 H.264 编码

         @param enable true - 在H.265编码失败时，自动降级到H.264编码，false - 不支持H.265编码自动降级，默认true
         @return true 成功，false 失败        
         @attention 支持版本：6.3.0 及以上。
         @attention 在推流前调用本接口。
         @attention 推 H.265 流时是否自动降级到 H.264 编码的开关；开启自动降级后，当不支持 H.265 编码或编码 H.265 失败时，SDK 内部会尝试降级使用 H.264 编码进行推流；关闭自动降级后，当不支持 H.265 编码或编码 H.265 失败时，直接推流失败。在推流前调用本接口。
         @attention 当推流过程中发生从 H.265 降级到 H.264 编码时，如果正在本地视频录制或云录制，会导致产生多个录制文件，需要对此做处理。
         */
        ZEGO_API bool EnableH265EncodeFallback(bool enable);
    
        /**
         是否支持指定视频编码
          
         @param codecID  编码ID
         @return true - 支持指定视频编码，false - 不支持指定视频编码
         @attention 支持版本：6.3.0 及以上。
         @attention 必须在InitSDK成功后才能调用本接口
         @attention 是否支持指定视频编码，主要取决于以下几个方面：硬件型号是否支持硬编、硬件型号性能是否支持软编、SDK 是否带该编码模块。
         @attention 建议用户在进行 H.265编码 推流前，先调用本接口获取 H.265 编码支持能力，如果不支持，可以采用其他编码进行推流，比如 H.264。
         */
        ZEGO_API bool IsVideoEncoderSupported(AV::ZegoVideoCodecAvc codecID);

        /**
        为某个推流道通单独指定推流是否校准网络时间，用于混流时各路流的对齐

        @param alignment 1设置，0 关闭
        @param idx 推流通道索引，默认主通道
        @return 0 表示成功，-1 为失败
        @attention InitSDK 后, 推流前后都可设置, 此方法依赖网络时间模块，且需要调用推流接口StartPublishingWithParams中推流，且参数 ZegoPublishStreamParams.forceSynchronousNetworkTime 为1开启网络时间同步
        */

        ZEGO_API int SetStreamAlignmentProperty(int alignment, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

#if TARGET_OS_IPHONE
        /**
         在 iPadOS 14 下，基于一些系统因素，导致 AVAudioSession 的实际状态与设置参数不一致，从而出现音量变小的情况，此时，可通过调用本接口恢复音量。
        */
        ZEGO_API void RecoverAudioSessionMode();


        /**
         设置预览的截图配置
         
         @attention 推流前或者推流过程中设置都有效，当次推流有效。
         @param cropRect 裁剪坐标
         @param originalSize 原始图像尺寸
         @param idx 推流通道号，参见 AV::PublishChannelIndex，默认值 AV::PUBLISH_CHN_MAIN
         */
        ZEGO_API void SetPreviewCropRect(AV::ZegoRect cropRect, AV::ZegoSize originalSize, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);
#endif // TARGET_OS_IOS

        /**
         设置视频编码模式
         @param mode 视频编码模式，详见 AV::ZegoVideoEncodeMode。默认是 AV::ZEGO_VIDEO_ENCODE_MODE_LOW_DELAY。
         @param idx 推流通道，默认为 AV::PUBLISH_CHN_MAIN
         @note 初始化 SDK 之后，推流之前设置有效，在开始推流后设置，下次推流生效。
         @note AV::ZEGO_VIDEO_ENCODE_MODE_LOW_BITRATE 模式只在使用 x264 软件编码的情况下生效，需要通过 ZEGO 的特殊渠道发包才能指定 x264 编码器。
         */
        ZEGO_API void SetVideoEncodeMode(AV::ZegoVideoEncodeMode mode, AV::PublishChannelIndex idx = AV::PUBLISH_CHN_MAIN);

        /**
         设置检测语音状态的回调
         
         @param callback 回调指针
         */
        ZEGO_API void SetAudioVADStableStateCallback(AV::IZegoCheckAudioVADCallback* callback);

        /**
         开启采集后的音频数据语音状态检测（包括麦克风采集和外部音频采集）
         
         @param enable 是否开启
         @note 检测周期是 3 秒钟
         @note 推流前后设置都有效。预览或者推流后开始回调
         */
        ZEGO_API void EnableCapturedAudioVADStableStateMonitor(bool enable);

        /**
         开启音频外部预处理后的音频数据的语音状态检测

         @param enable 是否开启
         @note 检测周期是 3 秒钟
         @note 推流前后设置都有效。预览或者推流后开始回调
         */
        ZEGO_API void EnableAudioPrepVADStableStateMonitor(bool enable);
    
    }
}
#endif /* LiveRoom_Publisher_h */
