//
//  zego-api-audio-observer.h
//

#ifndef zego_api_audio_observer_h
#define zego_api_audio_observer_h
#include "zego-api-defines.h"

namespace ZEGO
{
    namespace AUDIO_OBSERVER
    {

        /* * 音频源类型 */
        enum ZegoAudioObserverSource
        {
            /** 采集音频 */
            ZEGOAPI_AUDIO_OBSERVER_CAPTURE       = 0x01,
            /** 播放音频 */
            ZEGOAPI_AUDIO_OBSERVER_PLAYBACK      = 0x02,
            /** 采集与播放混音音频 */
            ZEGOAPI_AUDIO_OBSERVER_MIX           = 0x04,
            /** 系统播放和麦克采集的混音,仅支持Windows */
            ZEGOAPI_AUDIO_OBSERVER_SYSPLAY_AND_MIC = 0x08,
            /** 系统播放(不含拉流)的声音，仅支持 Windows */
            ZEGOAPI_AUDIO_OBSERVER_SYSPLAY_NO_STREAM = 0x10
        };

        /**
         音频观察器回调
         */
        class   IZegoAudioObserverCallback
        {
        public:
            /**
             获取采集的音频数据，数据已经经过3A处理
             
             @param data 音频数据，PCM格式
             @param dataLen 音频数据长度
             @param sampleRate 采样率，不固定，以当前值为准
             @param numChannels 通道数量，单通道
             @param bitDepth 位深度，16 bit
             @attention 包括本地麦克风采集、外部混音等音频类型
             @attention 同步回调，不能做耗时操作。拷贝数据后，切换线程再处理数据
             @note 依赖于 SetAudioObserverCallback 与 StartAudioObserver 的调用
             */
            virtual void OnCapturedAudioData(const unsigned char *data, int dataLen, int sampleRate, int numChannels, int bitDepth) {}

            /**
             获取播放的音频数据，数据已经经过3A处理
             
             @param data 音频数据，PCM格式
             @param dataLen 音频数据长度
             @param sampleRate 采样率，不固定，以当前值为准
             @param numChannels 通道数量，单通道
             @param bitDepth 位深度，16 bit
             @attention 包括拉流、媒体播放等音频类型
             @attention 同步回调，不能做耗时操作。拷贝数据后，切换线程再处理数据
             @note 依赖于 SetAudioObserverCallback 与 StartAudioObserver 的调用
             */
            virtual void OnPlaybackAudioData(const unsigned char *data, int dataLen, int sampleRate, int numChannels, int bitDepth) {}

            /**
             获取采集与播放混音后的音频数据，数据已经经过3A处理
             
             @param data 音频数据，PCM格式
             @param dataLen 音频数据长度
             @param sampleRate 采样率，不固定，以当前值为准
             @param numChannels 通道数量，单通道
             @param bitDepth 位深度，16 bit
             @attention 包括本地麦克风采集、外部混音、拉流、媒体播放等音频类型
             @attention 同步回调，不能做耗时操作。拷贝数据后，切换线程再处理数据
             @note 依赖于 SetAudioObserverCallback 与 StartAudioObserver 的调用
             */
            virtual void OnMixAudioData(const unsigned char *data, int dataLen, int sampleRate, int numChannels, int bitDepth) {}

            /**
             获取系统播放和麦克采集的混音，麦克采集的数据已经经过3A处理
             
             @param data 音频数据，PCM格式
             @param dataLen 音频数据长度
             @param sampleRate 采样率，不固定，以当前值为准
             @param numChannels 通道数量，单通道
             @param bitDepth 位深度，16 bit
             @attention 包括本地麦克风采集、系统播放等音频类型
             @attention 同步回调，不能做耗时操作。拷贝数据后，切换线程再处理数据。
             @note 依赖于 SetAudioObserverCallback 与 StartAudioObserver 的调用
             */
            virtual void OnSysplayAndMicData(const unsigned char *data, int dataLen, int sampleRate, int numChannels, int bitDepth) {}

            /**
             获取系统播放(不含拉流)的声音，数据已经经过3A处理
             
             @param data 音频数据，PCM格式
             @param dataLen 音频数据长度
             @param sampleRate 采样率，不固定，以当前值为准
             @param numChannels 通道数量，单通道
             @param bitDepth 位深度，16 bit
             @attention 包括系统播放(不含拉流)音频类型
             @attention 同步回调，不能做耗时操作。拷贝数据后，切换线程再处理数据。
             @note 依赖于 SetAudioObserverCallback 与 StartAudioObserver 的调用
             */
            virtual void OnSysplayNoStreamData(const unsigned char *data, int dataLen, int sampleRate, int numChannels, int bitDepth) {}


            /**
             错误回调
             @param error 错误码
             */
            virtual void OnAudioObserverError(int errorCode) = 0;
            
            virtual ~IZegoAudioObserverCallback() {}
        };

        /**
         设置音频观察器回调
         
         @param cb  音频观察器回调
         @return true 表示设置回调成功；false 表示设置回调失败。
         @note 调用StartAudioObserver成功后才会有回调
         @note 必须在初始化 SDK 后调用
         */
        ZEGOAVKIT_API bool SetAudioObserverCallback(IZegoAudioObserverCallback* cb);
    
        /**
         启动音频观察器，旁路回调相应的音频数据，该数据已经过3A处理，方便用户对相关音频数据进行录制、分析等自定义处理
         
         @param source 关注的音频源类型，只有关注了该类型的音频才会有相应的回调，详见ZegoAudioObserverSource
         @param sampleRate 音频采样率，包括8000, 16000, 22050, 24000, 32000, 44100, 48000
         @param numOfChannels 声道数，1：单声道，2：双声道
         @return true 表示调用成功，能收到音频观察器回调；false 表示调用失败，不能收到音频观察器回调。
         @note 1. 必须在初始化 SDK 后调用
         @note 2. 该模块与AV::EnableSelectedAudioRecord不兼容
         @note 3. 当source包含 ZEGOAPI_AUDIO_OBSERVER_CAPTURE 时，对应音频数据通过 OnCapturedAudioData 获取
         @note 4. 当source包含 ZEGOAPI_AUDIO_OBSERVER_PLAYBACK 时，对应音频数据通过 OnPlaybackAudioData 获取
         @note 5. 当source包含 ZEGOAPI_AUDIO_OBSERVER_MIX 时，对应音频数据通过 OnMixAudioData 获取
         @note 6. Windows下，当source包含 ZEGOAPI_AUDIO_OBSERVER_SYSPLAY_AND_MIC 时，对应音频数据通过 OnSysplayAndMicData 获取
		 @note 7. Windows下，当source包含 ZEGOAPI_AUDIO_OBSERVER_SYSPLAY_NO_STREAM 时，对应音频数据通过 OnSysplayNoStreamData 获取
		 @note 8. Windows下，ZEGOAPI_AUDIO_OBSERVER_SYSPLAY_AND_MIC、ZEGOAPI_AUDIO_OBSERVER_SYSPLAY_NO_STREAM 仅能开启一种
         */
        ZEGOAVKIT_API bool StartAudioObserver(int source, int sampleRate = 44100, int numOfChannels = 1);

        /**
         停止音频观察器，不会再回调相应的音频数据

         @note 必须在初始化 SDK 后调用
         */
        ZEGOAVKIT_API void StopAudioObserver();
           
    }
}

#endif /* zego_api_audio_observer_h */
